"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = require("path");

var _fs = _interopRequireDefault(require("fs"));

var _es6Promisify = _interopRequireDefault(require("es6-promisify"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const stat = (0, _es6Promisify.default)(_fs.default.stat);

class FileResource {
  constructor(file) {
    this.file = (0, _path.normalize)((0, _path.resolve)(file));
  }

  async lastModified() {
    try {
      const {
        mtimeMs,
        mtime
      } = await stat(this.file);
      return mtimeMs != null ? mtimeMs : mtime.getTime();
    } catch (err) {
      if (err.code === 'ENOENT') return null;
      throw err;
    }
  }

  updateHash(hash) {
    return new Promise((resolve, reject) => {
      const input = _fs.default.createReadStream(this.file);

      input.on('readable', () => {
        const data = input.read();
        if (data) hash.update(data);else resolve();
      });
      input.on('error', reject);
    });
  }

  toString() {
    return (0, _path.relative)(process.cwd(), this.file);
  }

}

exports.default = FileResource;
module.exports = exports.default;