"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Rule = _interopRequireDefault(require("./Rule"));

var _crypto = require("crypto");

var _fs = _interopRequireDefault(require("fs"));

var _Verbosity = _interopRequireDefault(require("./Verbosity"));

var _es6Promisify = _interopRequireDefault(require("es6-promisify"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class HashRule extends _Rule.default {
  constructor(props) {
    super(props);

    _defineProperty(this, "_make", async (context) => {
      const hash = (0, _crypto.createHash)(this.hashAlgorithm);
      const {
        targets,
        promake,
        recipe
      } = this;
      const prerequisites = this.prerequisites;
      const [target] = targets;
      if (!target) throw new Error('missing target');
      const {
        file
      } = target;
      if (typeof file !== 'string') throw new Error('missing targets[0].file');

      for (let prerequisite of prerequisites) {
        await promake._make(prerequisite, context);
        await prerequisite.updateHash(hash);
      }

      const digest = hash.digest('hex');
      let lastDigest;

      try {
        lastDigest = (await (0, _es6Promisify.default)(_fs.default.readFile)(file, 'utf8')).trim();
      } catch (error) {
        if (error.code !== 'ENOENT') throw error;
      }

      if (lastDigest === digest) {
        promake.log(_Verbosity.default.DEFAULT, 'Nothing to be done for', this.toString(), 'hash:', lastDigest);
        return;
      }

      if (recipe) {
        promake.log(_Verbosity.default.DEFAULT, 'Making', this.toString(), 'hash:', lastDigest);
        await recipe(this);
      }

      await (0, _es6Promisify.default)(_fs.default.writeFile)(file, digest, 'utf8');
      this.lastFinishTime = Date.now();
    });

    const {
      targets: _targets,
      prerequisites: _prerequisites
    } = props;

    if (_targets.length !== 1 || typeof _targets[0].file !== 'string') {
      throw new Error('targets must contain a single FileResource');
    }

    for (let prerequisite of _prerequisites) {
      if (typeof prerequisite.updateHash !== 'function') throw new Error(`prerequisite lacks an updateHash function: ${String(prerequisite)}`);
    }
  }

}

exports.default = HashRule;
module.exports = exports.default;