"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _Rule = _interopRequireDefault(require("./Rule"));

var _TaskResource = _interopRequireDefault(require("./TaskResource"));

var _Verbosity = _interopRequireDefault(require("./Verbosity"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class LastModifiedRule extends _Rule.default {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "_make", async (context) => {
      const {
        targets,
        promake,
        prerequisites,
        recipe
      } = this;
      const targetTimes = await Promise.all(targets.map(target => target.lastModified()));
      const prerequisiteTimes = [];

      if (targets.length === 1 && targets[0] instanceof _TaskResource.default && !recipe) {
        promake.log(_Verbosity.default.DEFAULT, 'Making', this.toString());
      }

      for (let prerequisite of prerequisites) prerequisiteTimes.push(await promake._make(prerequisite, context));

      const finiteTargetTimes = targetTimes.filter(Number.isFinite);

      if (finiteTargetTimes.length === targetTimes.length && !this.runAtLeastOnce) {
        const finitePrerequisiteTimes = prerequisiteTimes.filter(Number.isFinite);
        const minTargetTime = Math.min(...finiteTargetTimes);
        const maxPrerequisiteTime = Math.max(...finitePrerequisiteTimes);

        if (!prerequisites.length || minTargetTime > maxPrerequisiteTime) {
          promake.log(_Verbosity.default.DEFAULT, 'Nothing to be done for', this.toString());
          return;
        }
      }

      if (recipe) {
        promake.log(_Verbosity.default.DEFAULT, 'Making', this.toString());
        await recipe(this);
      }

      this.lastFinishTime = Date.now();
    });
  }

}

exports.default = LastModifiedRule;
module.exports = exports.default;