"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var t = _interopRequireWildcard(require("typed-validators"));

var _Resource = require("./Resource");

var _FileResource = _interopRequireDefault(require("./FileResource"));

var _Rule = _interopRequireWildcard(require("./Rule"));

var _LastModifiedRule = _interopRequireDefault(require("./LastModifiedRule"));

var _HashRule = _interopRequireDefault(require("./HashRule"));

var _TaskResource = _interopRequireDefault(require("./TaskResource"));

var _promisifyChildProcess = require("promisify-child-process");

var _chalk = _interopRequireDefault(require("chalk"));

var _Verbosity = _interopRequireDefault(require("./Verbosity"));

var _lodash = _interopRequireDefault(require("lodash.padend"));

var _ExecutionContext = _interopRequireDefault(require("./ExecutionContext"));

var _handleChildProcessCleanup = _interopRequireDefault(require("./handleChildProcessCleanup"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// const ResourcesType = (reify: Type<Array<string | BasicResource> | string | BasicResource>)
// const PrerequisitesType = (reify: Type<Array<string | BasicResource | Rule> | string | BasicResource | Rule>)
// const HashPrerequisitesType = (reify: Type<Array<string | BasicHashResource | Rule> | string | BasicHashResource | Rule>)
const ResourcesType = t.alias('Resources', t.oneOf(t.array(t.oneOf(t.string(), t.ref(() => _Resource.ResourceType))), t.string(), t.ref(() => _Resource.ResourceType)));
const BasicResourceType = t.alias('BasicResource', t.record(t.any(), t.any()));
BasicResourceType.addConstraint(resource => {
  if (typeof resource.lastModified !== 'function' || resource.lastModified.length !== 0) return 'must have a lastModified method that takes no arguments';
});
const BasicHashResourceType = t.alias('BasicHashResource', t.record(t.any(), t.any()));
BasicHashResourceType.addConstraint(resource => {
  if (typeof resource.updateHash !== 'function' || resource.updateHash.length !== 1) return 'must have a updateHash method that takes one argument';
});
const PrerequisitesType = t.alias('Prerequisites', t.oneOf(t.array(t.oneOf(t.string(), t.ref(() => BasicResourceType), t.ref(() => _Rule.RuleType))), t.string(), t.ref(() => BasicResourceType), t.ref(() => _Rule.RuleType)));
const HashPrerequisitesType = t.alias('Prerequisites', t.oneOf(t.array(t.oneOf(t.string(), t.ref(() => BasicHashResourceType), t.ref(() => _Rule.RuleType))), t.string(), t.ref(() => BasicHashResourceType), t.ref(() => _Rule.RuleType)));

class Promake {
  constructor() {
    _defineProperty(this, "fileResources", new Map());

    _defineProperty(this, "taskResources", new Map());

    _defineProperty(this, "rules", new Map());

    _defineProperty(this, "verbosity", _Verbosity.default.DEFAULT);

    _defineProperty(this, "_normalizeResource", resource => {
      if (typeof resource === 'string') {
        const normalized = new _FileResource.default(resource);
        let file = this.fileResources.get(normalized.file);
        if (!file) this.fileResources.set(normalized.file, file = normalized);
        return file;
      }

      return resource;
    });

    _defineProperty(this, "_normalizePrerequisite", resource => {
      if (resource instanceof _Rule.default) return resource.targets;
      return this._normalizeResource(resource);
    });

    _defineProperty(this, "_normalizeResources", resources => {
      ResourcesType.assert(resources, '', ['resources']);
      return Array.isArray(resources) ? resources.map(this._normalizeResource) : [this._normalizeResource(resources)];
    });

    _defineProperty(this, "_normalizePrerequisites", prerequisites => {
      PrerequisitesType.assert(prerequisites, '', ['prerequisites']);
      if (Array.isArray(prerequisites)) return [].concat(...prerequisites.map(this._normalizePrerequisite));

      const normalized = this._normalizePrerequisite(prerequisites);

      return Array.isArray(normalized) ? normalized : [normalized];
    });

    _defineProperty(this, "_normalizeHashPrerequisites", prerequisites => {
      HashPrerequisitesType.assert(prerequisites, '', ['prerequisites']);
      if (Array.isArray(prerequisites)) return [].concat(...prerequisites.map(this._normalizePrerequisite));

      const normalized = this._normalizePrerequisite(prerequisites);

      return Array.isArray(normalized) ? normalized : [normalized];
    });

    _defineProperty(this, "_normalizeName", name => {
      const result = this.taskResources.get(name) || this.fileResources.get(new _FileResource.default(name).file);
      if (!result) throw new Error(`no task or file found for ${name}`);
      return result;
    });

    _defineProperty(this, "_make", async (resource, context) => {
      const rule = this.rules.get(resource);

      if (!rule) {
        if (typeof resource.lastModified === 'function') {
          let time = await resource.lastModified();
          if (Number.isFinite(time)) return time;
        }

        throw new Error(`No rule found to make ${resource.toString()}`);
      }

      await rule.make(context);
      return resource.lastModified();
    });

    _defineProperty(this, "make", async (target, context = new _ExecutionContext.default()) => {
      if (typeof target === 'string') target = this._normalizeName(target);
      await this._make(target, context);
    });

    _defineProperty(this, "log", (verbosity, ...args) => {
      if (this.verbosity >= verbosity) console.error(_chalk.default.bold('[promake]'), ...args); // eslint-disable-line no-console
    });

    _defineProperty(this, "logStream", (verbosity, stream) => {
      if (this.verbosity >= verbosity) stream.pipe(process.stderr);
    });

    _defineProperty(this, "_logChildProcess", child => {
      if (this.verbosity <= _Verbosity.default.QUIET) return;
      if (child.stdout) child.stdout.pipe(process.stdout);
      if (child.stderr) child.stderr.pipe(process.stderr);
    });

    _defineProperty(this, "exec", (command, options = {}) => {
      const child = (0, _promisifyChildProcess.exec)(command, options);
      (0, _handleChildProcessCleanup.default)(this, child);
      if (this.verbosity >= _Verbosity.default.DEFAULT) console.error(_chalk.default.gray('$'), _chalk.default.gray(command)); // eslint-disable-line no-console

      this._logChildProcess(child);

      return child;
    });

    _defineProperty(this, "spawn", (command, argsOrOptions, options) => {
      const args = Array.isArray(argsOrOptions) ? argsOrOptions : [];

      if (argsOrOptions instanceof Object && !Array.isArray(argsOrOptions)) {
        options = argsOrOptions;
      }

      if (!options) options = {};

      function formatArg(arg) {
        if (/^[-_a-z0-9=./]+$/i.test(arg)) return _chalk.default.gray(arg);
        return _chalk.default.gray(`'${arg.replace(/'/g, "'\\''")}'`);
      }

      const child = (0, _promisifyChildProcess.spawn)(command, args, options);
      (0, _handleChildProcessCleanup.default)(this, child);
      if (this.verbosity >= _Verbosity.default.DEFAULT) // eslint-disable-next-line no-console
        console.error(_chalk.default.gray('$'), _chalk.default.gray(command), ...args.map(formatArg));

      this._logChildProcess(child);

      return child;
    });

    _defineProperty(this, "rule", (targets, prerequisites, recipe, options) => {
      if (typeof prerequisites === 'function') {
        options = recipe;
        recipe = prerequisites;
        prerequisites = null;
      }

      if (!prerequisites && !recipe) {
        if (Array.isArray(targets)) throw new Error('You must pass only one target when looking up a rule');

        const target = this._normalizeResource(targets);

        const rule = this.rules.get(target);
        if (!rule) throw new Error(`No rule found for ${target.toString()}`);
        return rule;
      }

      const rule = new _LastModifiedRule.default({
        promake: this,
        targets: this._normalizeResources(targets),
        prerequisites: prerequisites ? this._normalizePrerequisites(prerequisites) : [],
        recipe,
        runAtLeastOnce: Boolean(options && options.runAtLeastOnce)
      });

      for (let target of rule.targets) {
        if (this.rules.has(target)) throw new Error(`A rule for ${target.toString()} already exists`);
        this.rules.set(target, rule);
      }

      return rule;
    });

    _defineProperty(this, "hashRule", (algorithm, target, prerequisites, recipe, options) => {
      const finalTarget = this._normalizeResource(target);

      if (typeof prerequisites === 'function') {
        options = recipe;
        recipe = prerequisites;
        prerequisites = null;
      }

      if (!prerequisites && !recipe) {
        const rule = this.rules.get(finalTarget);
        if (!rule) throw new Error(`No rule found for ${target.toString()}`);
        if (!(rule instanceof _HashRule.default)) throw new Error(`Rule for target is not a HashRule`);
        return rule;
      }

      const rule = new _HashRule.default({
        hashAlgorithm: algorithm,
        promake: this,
        targets: [finalTarget],
        prerequisites: prerequisites ? this._normalizeHashPrerequisites(prerequisites) : [],
        recipe,
        runAtLeastOnce: Boolean(options && options.runAtLeastOnce)
      });

      for (let target of rule.targets) {
        if (this.rules.has(target)) throw new Error(`A rule for ${target.toString()} already exists`);
        this.rules.set(target, rule);
      }

      return rule;
    });

    _defineProperty(this, "task", (name, prerequisites, recipe) => {
      if (prerequisites instanceof Function) {
        recipe = prerequisites;
        prerequisites = null;
      }

      if (!prerequisites && !recipe) {
        const target = this.taskResources.get(name);
        const result = target ? this.rules.get(target) : null;
        if (!result) throw new Error(`No task named ${name} exists`);
        return result;
      }

      if (this.taskResources.has(name)) throw new Error(`A task named ${name} already exists`);
      const target = new _TaskResource.default({
        name,
        promake: this
      });
      this.taskResources.set(name, target);
      const rule = new _LastModifiedRule.default({
        promake: this,
        targets: [target],
        prerequisites: prerequisites ? this._normalizePrerequisites(prerequisites) : [],
        recipe
      });
      this.rules.set(target, rule);
      return rule;
    });

    _defineProperty(this, "printUsage", () => {
      const {
        version,
        homepage
      } = require('./packageInfo');

      const tasks = [...this.taskResources.keys()].sort();
      const taskColumnWidth = Math.max(16, ...tasks.map(name => name.length)) + 2;
      process.stderr.write(`promake CLI, version ${version}
${homepage}/tree/v${version}

Usage:
  ./<script> [options...] [tasks...]

Options:
  -q, --quiet       suppress output
  -v, --verbose     verbose output

Tasks:
  ${tasks.map(task => `${(0, _lodash.default)(task, taskColumnWidth)}${this.task(task).description() || ''}`).join('\n  ') || '(No tasks defined)'}
`);
    });

    _defineProperty(this, "cli", async (argv = process.argv, options = {}) => {
      let ruleArgsMode = false;

      try {
        argv = argv.slice(2);
        const targets = [];
        let lastTarget;

        for (let i = 0; i < argv.length; i++) {
          if (ruleArgsMode) {
            if (argv[i] === '--') {
              ruleArgsMode = false;
            } else if (lastTarget) {
              this.rule(lastTarget).args.push(argv[i].replace(/^--(-+)$/, '$1'));
            }
          } else if (argv[i].startsWith('-')) {
            if (lastTarget) {
              if (argv[i] === '--') {
                this.rule(lastTarget).args = [];
                ruleArgsMode = true;
              } else {
                this.rule(lastTarget).args.push(argv[i]);
              }
            } else {
              switch (argv[i]) {
                case '-q':
                case '--quiet':
                  this.verbosity = _Verbosity.default.QUIET;
                  break;

                case '-v':
                case '--verbose':
                  this.verbosity = _Verbosity.default.HIGH;
                  break;

                default:
                  throw new Error(`unrecognized option: ${argv[i]}`);
              }
            }
          } else {
            try {
              targets.push(lastTarget = this._normalizeName(argv[i]));
            } catch (error) {
              if (lastTarget) {
                this.rule(lastTarget).args.push(argv[i]);
              } else {
                throw error;
              }
            }
          }
        }

        if (!targets.length) this.printUsage();

        for (let target of targets) await this.make(target);

        if (options.exit !== false) process.exit(0);
      } catch (error) {
        if (options.exit !== false) {
          this.log(_Verbosity.default.DEFAULT, error.stack);
          process.exit(1);
        }

        throw error;
      }
    });
  }

  static checkResource(resource, path) {
    if (typeof resource === 'string') return;

    if (!(resource instanceof Object)) {
      throw new Error('Resource must be a string or conform to the Resource interface');
    }

    if (typeof resource.lastModified !== 'function') {
      throw new Error('resource ');
    }
  }

}

exports.default = Promake;

_defineProperty(Promake, "VERBOSITY", _Verbosity.default);

module.exports = exports.default;