"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = handleChildProcessCleanup;

var _Verbosity = _interopRequireDefault(require("./Verbosity"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @prettier
 * 
 */
let killCount = 0;
const runningProcesses = new Set();

function handleChildProcessCleanup(promake, child) {
  function die() {
    process.off('SIGINT', killHandler);
    process.kill(process.pid, 'SIGINT');
  }

  function killHandler() {
    killCount++;
    if (killCount === 1) promake.log(_Verbosity.default.DEFAULT, 'cleaning up, please wait...');else promake.log(_Verbosity.default.QUIET, 'got second SIGINT, killing spawned processes with SIGKILL');

    for (const child of runningProcesses) child.kill(killCount > 1 ? 'SIGKILL' : 'SIGINT');

    if (killCount > 1) die();
  }

  if (!runningProcesses.size) {
    process.on('SIGINT', killHandler);
  }

  runningProcesses.add(child);

  const cleanupChild = () => {
    child.removeListener('close', cleanupChild);
    child.removeListener('error', cleanupChild);
    runningProcesses.delete(child);

    if (!runningProcesses.size) {
      process.off('SIGINT', killHandler);
      if (killCount) die();
    }
  };

  child.once('close', cleanupChild);
  child.once('error', cleanupChild);
}

module.exports = exports.default;